-- 指定位置刷箱子，箱子生产指定物体
local function SpawnBlock( a_World, jval )

    local blocktype = jval["blocktype"];
    local blockmeta = jval["blockmeta"];-- 0x02 / 0x03 / 0x04 / 0x05
    local x  = jval["x"];
    local y  = jval["y"];
    local z  = jval["z"];

    local block = a_World:GetBlockEntity(x, y, z);
    if block == nil then
        a_World:SetBlock(x, y, z, blocktype, blockmeta, true);
        block = a_World:GetBlockEntity(x, y, z);
    end

    local jitems = jval["items"];
    for k, v in ipairs(jitems) do

        local id        = v["id"];          --刷出的物品数量
        local count     = v["count"];       --刷出的物品数量
        local meta      = v["meta"];        --刷出物品的损坏度
        local slotnum   = v["slotnum"];     --箱子里的第几格
        local item      = cItem(id, count, meta);
        block:SetSlot(slotnum, item);

    end

    return true;

end

-- 指定位置物品
local function SpawnItem( a_World, jval )

    local x  = jval["x"];
    local y  = jval["y"];
    local z  = jval["z"];

    local id        = jval["id"];          --刷出的物品数量
    local count     = jval["count"];       --刷出的物品数量 
    local meta      = jval["meta"];        --刷出物品的损坏度

    local Items = cItems()
    Items:Add(cItem(id, count, meta))
    World:SpawnItemPickups(Items, x, y, z, 0, 0, 0, false)

    return true;

end

-- 刷怪
local function SpawnMob( a_World, jval )

    local posx    = jval["x"];
    local posy    = jval["y"];
    local posz    = jval["z"];
    local mobtype = jval["mobtype"];
    local bisbaby = jval["isbaby"];
    local ai      = jval["ai"];
    local scale   = jval["scale"]
    local count   = jval["count"]
    local burnsindaylight  = jval["burnsindaylight"]

    local health           = jval["health"]
    local attack_damage    = jval["attack_damage"]

    local drop_id    = jval["drop_id"] or 0

    local spawnRadius = jval["spawn_radius"] or 5

    local i = 0;
    math.randomseed(os.time())
    while (i < count)
    do
        i = i + 1;
        local offset_x = math.random(-spawnRadius, spawnRadius);
        local offset_z = math.random(-spawnRadius, spawnRadius);
        
        monster = a_World:SpawnMobBy(posx + offset_x, posy, posz + offset_z, mobtype, bisbaby, scale, ai, drop_id);
        if monster ~= nil then
            if health ~= nil then
                monster:SetMaxHealth(health)
                monster:SetHealth(health)
            end

            if attack_damage ~= nil then
                monster:SetAttackDamage(attack_damage)
            end

            if burnsindaylight ~= nil then
                monster:SetBurnsInDaylight(burnsindaylight)
            end
            monster:SetNeedStay(true);         
        end
    end
    return true

end

local function  ActiveTask(a_World, jval)

    local taskid = jval["taskid"];
    return a_World:ActiveTask(taskid);

end

local function  EndTask(a_World, jval)

    local taskid = jval["taskid"];
    return a_World:EndTask(taskid);

end


local function  ReachTask(a_World, jval)

    local taskid = jval["taskid"];
    return a_World:ReachTask(taskid);

end

local function  StartTask(a_World, jval)

    local jtaskids = jval["taskids"];
    for k, taskid in ipairs(jtaskids) do
        if a_World:StartTask(taskid) then
            return true;
        end
    end
    return false

end

-- 增加物品
local function AddItem( a_World, jval )
    local jitems = jval["items"];
    for k,v in ipairs(jitems) do
        local id    = v["id"];
        local meta  = v["meta"]
        local count = v["count"]
        a_World:AddItemToAllPlayer(id, meta, count);
    end

    local tb = {};
    tb["name"]      = "AddItem";
    tb["items"]     = jval["items"];
    local param = cJson:Serialize(tb)
    a_World:BroadcastPluginMessage(param);
    return true
end

-- 解锁物品
local function UnlockItem( a_World, jval )
    local tb = {};
    tb["name"]     = "UnlockItem";
    tb["items"]    = jval["items"];

    local param = cJson:Serialize(tb)
    a_World:BroadcastPluginMessage(param);
    return true;
end

-- 跟随
local function Follow( a_World, jval )
    local follower_id       = jval["follower_id"]; --跟随都id，现支持npcid
    local be_follow_id      = jval["be_follow_id"]; -- 被跟随者的id，-1为玩家（现仅支持玩家）

    local pause_follow_dis      = jval["pause_follow_dis"] or 2.0   --暂停跟随的判定距离，小于这个距离将暂停跟随，默认为2.0
    local continue_follow_dis   = jval["continue_follow_dis"] or 4.0   --继续跟随的判定距离，当暂停跟随时，将等待与被跟随者的距离大于这个距离才继续跟随，默认为4.0
    local speed_up_follow_dis   = jval["speed_up_follow_dis"] or 6.0  --加速跟随的判定距离,当与被跟随者的距离大于这个距离时加速跟随，默认为6.0
    local speed_up_rate         = jval["speed_up_rate"] or 1.0   --加速跟随时，加速的倍率，默认为2.0,加速200%
    local teleportation_dis     = jval["teleportation_dis"] or 8.0    --瞬间移动的判定距离,默认8.0
    -- 注意：应该满足以下条件     pause_follow_dis <= continue_follow_dis <= teleportation_dis

    local follower = follower_id and a_World:GetNpcFromId(follower_id)
    if follower == nil then
        LOGWARN(string.format("Follow: Can't find the follower (%s)", follower_id))
        return
    end

    a_World:PluginFollow(follower, pause_follow_dis, continue_follow_dis, speed_up_follow_dis, teleportation_dis, speed_up_rate)
end

-- 停止跟随
local function FollowEnd( a_World, jval )
    local follower_id     = jval["follower_id"]; --跟随都id，现支持npcid
    local be_follow_id    = jval["be_follow_id"]; -- 被跟随者的id，-1为玩家
    
    local follower = follower_id and a_World:GetNpcFromId(follower_id)
    if follower == nil then
        LOGWARN(string.format("FollowEnd: Can't find the follower (%s)", follower_id))
        return
    end

    follower:FollowEnd();
end

-- 结束跟随并移动到指定点
local function FollowEndAndMoveToPosion( a_World, jval )
    local npc_id    = jval["npc_id"];
    local x         = jval["x"]; 
    local y         = jval["y"];
    local z         = jval["z"];

    local npc = a_World:GetNpcFromId(npc_id)
    if npc == nil then
        LOGWARN(string.format("FollowEndAndMoveToPosion: Can't find the npc (%s)", npc_id))
        return
    end
    npc:FollowEndAndMoveToPosion(x, y, z);
end

-- 增加蓝钻
local  function AddGold( a_World, jval )
    local tb = {};
    tb["name"]      = "AddGold";
    tb["count"]     = jval["count"];

    local param = cJson:Serialize(tb)
    a_World:BroadcastPluginMessage(param);
    return true;
end

-- 解锁皮肤
local  function UnlockSkin( a_World, jval )

    local tb = {};
    tb["name"]     = "UnlockSkin";
    tb["skins"]    = jval["skins"];
    
    local param = cJson:Serialize(tb)
    a_World:BroadcastPluginMessage(param);
    return true;

end

-- 跳转到对应的 stepindex   contentsindex contentindex
local  function SkipDialog( a_World, jval )
    local tb = {};
    tb["name"]              = "SkipDialog";
    tb["stepindex"]         = jval["stepindex"];
    tb["contentsindex"]     = jval["contentsindex"];
    tb["contentindex"]      = jval["contentindex"];
    local param = cJson:Serialize(tb)
    a_World:BroadcastPluginMessage(param);
    return true;

end

 -- Spawn a symbol Entity with a type in a position
local function SpawnSymbol(a_World, jval)
    local symbol_id     = jval["id"]
    local symbol_type   = jval["type"]
    local x             = jval["x"]
    local y             = jval["y"]
    local z             = jval["z"]  

    a_World:SpawnSymbolEntity(symbol_id, symbol_type, x, y, z);
    return true;
end
    
-- Destroy a symbol Entity with a type in a position
local function DestroySymbol(a_World, jval)
    local symbol_id     = jval["id"]
    a_World:DestroySymbolEntity(symbol_id)
    return true;
end
    
-- Destroy a symbol Entity with a type in a position
local function ModifySymbol(a_World, jval)
    local symbol_id     = jval["id"]
    local symbol_type   = jval["type"]
    a_World:ModifySymbolEntity(symbol_id, symbol_type)
    return true;
end

-- end game
local function PassTollgate(a_World, jval)

    local tb = {};
    tb["name"]     = "PassTollgate";
    tb["state"]    = jval["state"]
    
    local param = cJson:Serialize(tb)
    a_World:BroadcastPluginMessage(param);
    return true;
end

-- 指定的blockentity增加trigger
local function AddTriggerToBlockEntity( a_World, jval )

    local x  = jval["x"];
    local y  = jval["y"];
    local z  = jval["z"];

    local blockentity = a_World:GetBlockEntity(x, y, z);
    if blockentity == nil then
        return false;
    end

    local cf = cJson:Serialize(jval["trigger"])
    blockentity:LoadTriggerParam(cf);
    return true;

end

local function  SetSpawnpoint( a_World, jval )
    local id  = jval["spawnpointid"];
    a_World:SetLastSpawnPoint(id);
    return true;
end

local function InHandItem( a_World, jval  )
    local id        = jval["id"];          --刷出的物品数量
    local meta      = jval["meta"];        --刷出物品的损坏度
    local item = cItem(id, 1, meta)

    local position = 0;
    a_World:InHandItem(item, position);

    local tb = {};
    tb["name"]     = "InHandItem";
    tb["position"] = position;
    
    local param = cJson:Serialize(tb)
    a_World:BroadcastPluginMessage(param);
end

-- 给npc加个效果
local function AddEffectToNpc( a_World, jval  )
    local npc_id    = jval["npc_id"];
    local effect_id = jval["effect_id"];          -- 效果ID
    a_World:AddEffectToNpc(npc_id, effect_id);
end

-- 移除npc身上指定效果
local function RemoveEffectFromNpc( a_World, jval  )
    local npc_id    = jval["npc_id"];
    local effect_id = jval["effect_id"];          -- 效果ID
    a_World:RemoveEffectFromNpc(npc_id, effect_id);
end

-- 设置玩家多人乘骑能力，目前包括：上下船／控制船／上下矿车／控制矿车
local function SetPlayerSpeAbi( a_World, jval  )
    local player_ability    = jval["player_ability"];
    local enable            = jval["enable"];          
    a_World:BroadcastPlayerSpeAbi(player_ability, enable);
end

-- 修改barrier的激活状态
local function SetBarrierActivated( a_World, jval  )
    local id        = jval["id"];
    local activated = jval["activated"];
    a_World:SetBarrierActivated(id, activated); 
end

-- 修改NPC的名字
local function SetNPCName( a_World, jval  )
    local npc_id    = jval["npc_id"];
    local name      = jval["npc_name"];

    local npc = a_World:GetNpcFromId(npc_id)
    if npc == nil then
        LOGWARN(string.format("FollowEndAndMoveToPosion: Can't find the npc (%s)", npc_id))
        return
    end
    npc:SetCustomName(name);
end

--[[
接口:CreateEntity
说明:创建一个entity
参数：
    entity_type:entity类型，"minecrat"为矿车,"boat"为船
    x,y,z为创建的位置
    ]]
local function CreateEntity( a_World, jval  )
    local entity_type   = jval["entity_type"] --"minecrat","boat"
    local pos_x         = jval["x"]
    local pos_y         = jval["y"]
    local pos_z         = jval["z"]
    
    a_World:CreateEntity(entity_type, pos_x, pos_y, pos_z)
end

-- 修改NPC的位置
local function SetNPCPosition( a_World, jval  )
    local npc_id    = jval["npc_id"]
    local pos_x     = jval["x"]
    local pos_y     = jval["y"]
    local pos_z     = jval["z"]

    local npc = a_World:GetNpcFromId(npc_id)
    if npc == nil then
        LOGWARN(string.format("SetNPCPosition: Can't find the npc (%s)", npc_id))
        return
    end

    npc:SetPosition(pos_x, pos_y, pos_z)
end

-- 设置一块区域内的方块类型
local function SetAreaBlock( a_World, jval )

    local x_start   = jval["x"]
    local y_start   = jval["y"]
    local z_start   = jval["z"]
    local x_end     = x_start + jval["sizex"]
    local y_end     = y_start + jval["sizey"]
    local z_end     = z_start + jval["sizez"]

    local blocktype = jval["blocktype"];
    local blockmeta = jval["blockmeta"];-- 0x02 / 0x03 / 0x04 / 0x05

    if (x_start>x_end) then
        x_start, x_end = x_end, x_start
    end
    if (y_start>y_end) then
        x_start, x_end = x_end, x_start
    end
    if (z_start>z_end) then
        x_start, x_end = x_end, x_start
    end

    local Area = cBlockArea();
    Area:Create(x_end-x_start+1, y_end-y_start+1, z_end-z_start+1)
    Area:SetOrigin(x_start, y_start, z_start)
    for x=x_start,x_end,1 
    do
        for y=y_start,y_end,1 
        do
            for z=z_start,z_end,1 
            do
                Area:SetBlockType(x, y, z, blocktype)
                Area:SetBlockMeta(x, y, z, blockmeta)
            end
        end
    end

    a_World:WriteBlockArea(Area, x_start, y_start, z_start, 3)
end

-- 删除玩家背包里的物品
local function RemoveItem( a_World, jval )

    local id    = jval["id"];
    local meta  = jval["meta"];
    local count = jval["count"];
    a_World:RemoveItem(id, meta, count)
end

-- globle action func list
local action_func_list = {}
action_func_list["SpawnBlock"]                  = SpawnBlock
action_func_list["SpawnMob"]                    = SpawnMob
action_func_list["ActiveTask"]                  = ActiveTask
action_func_list["ReachTask"]                   = ReachTask
action_func_list["StartTask"]                   = StartTask
action_func_list["AddItem"]                     = AddItem
action_func_list["UnlockItem"]                  = UnlockItem
action_func_list["AddGold"]                     = AddGold
action_func_list["UnlockSkin"]                  = UnlockSkin
action_func_list["SpawnItem"]                   = SpawnItem
action_func_list["SpawnSymbol"]                 = SpawnSymbol
action_func_list["DestroySymbol"]               = DestroySymbol
action_func_list["ModifySymbol"]                = ModifySymbol
action_func_list["PassTollgate"]                = PassTollgate
action_func_list["AddTriggerToBlockEntity"]     = AddTriggerToBlockEntity
action_func_list["SetSpawnpoint"]               = SetSpawnpoint
action_func_list["SkipDialog"]                  = SkipDialog
action_func_list["EndTask"]                     = EndTask
action_func_list["Follow"]                      = Follow 
action_func_list["FollowEnd"]                   = FollowEnd 
action_func_list["InHandItem"]                  = InHandItem
action_func_list["AddEffectToNpc"]              = AddEffectToNpc
action_func_list["RemoveEffectFromNpc"]         = RemoveEffectFromNpc
action_func_list["SetPlayerSpeAbi"]             = SetPlayerSpeAbi
action_func_list["SetBarrierActivated"]         = SetBarrierActivated
action_func_list["FollowEndAndMoveToPosion"]    = FollowEndAndMoveToPosion
action_func_list["SetNPCName"]                  = SetNPCName
action_func_list["CreateEntity"]                = CreateEntity
action_func_list["SetNPCPosition"]              = SetNPCPosition
action_func_list["SetAreaBlock"]                = SetAreaBlock
action_func_list["RemoveItem"]                  = RemoveItem

-- task enter
function OnTask( task_name, a_World, param )
	local jval = cJson:Parse(param);

    func = action_func_list[task_name];
    if (func == nil) then
        return false;
    end
    return func(a_World, jval)
end
