#include "Uniforms.glsl"
#include "Samplers.glsl"
#include "Transform.glsl"
#include "ScreenPos.glsl"
#include "Lighting.glsl"
#include "Fog.glsl"
#include "MCHeader.glsl"
varying vec2 vTexCoord;
varying vec4 vScreenPos;
varying vec4 vWorldPos;
varying vec4 vColor;

//#if defined(CLOUD)
//uniform vec4 cCloudColorFactor;
//#endif

#if defined(FREQUENCYUVOFFSET)
uniform vec2  cFrequencyUVoffset;
//using to control vec2  uv begin and  vec2 uv end
uniform vec4  cUVMinMaxoffset;
varying vec2  vGoalUVoffset;

#if defined(SEGMENTUV)
uniform vec2  cSegmentUVNum;
#endif
#endif


varying vec3 color;

//SoSo by yuxiaolong define a fog color
varying  float vOffsetSpec;
varying vec4 vMCSpecColor;
varying vec3 vMCSpecFactor;
////soso by yuxiaolong
//const vec3 GammaV3=vec3(1.05,1.05,1.15);

void VS()
{
    mat4 modelMatrix = iModelMatrix;
    vec3 worldPos = GetWorldPos(modelMatrix);
    gl_Position = GetClipPos(worldPos);
    vWorldPos = vec4(worldPos, GetDepth(gl_Position));

#if defined(CLOUD)
    vColor=iColor;//*cCloudColorFactor;
    vTexCoord = GetTexCoord(iTexCoord);
#elif defined(STATIONARY_RAIN)
    vTexCoord = GetTexCoord(iTexCoord);
    vColor=iColor;
#elif defined(STATIONARY_ICON)
    vTexCoord = GetTexCoord(iTexCoord);
    vColor=iColor;
    vMCSpecColor =  MCGetSpecColor();
    vMCSpecFactor = MCGetSpecFactor();
    vOffsetSpec = fract(cElapsedTime/vMCSpecFactor.z);
#else
    vTexCoord = GetTexCoord(iTexCoord);
    vColor=iColor;
    vMCSpecColor =  MCGetSpecColor();
    vMCSpecFactor = MCGetSpecFactor();
    vOffsetSpec = fract(cElapsedTime/vMCSpecFactor.z);
#endif
    
#if defined(FREQUENCYUVOFFSET)
    vec2 uvSize = cUVMinMaxoffset.zw - cUVMinMaxoffset.xy;
    float goalUoffsetNormal = fract(cElapsedTime*cFrequencyUVoffset.x);
    float goalVoffsetNormal = fract(cElapsedTime*cFrequencyUVoffset.y);
#if defined(SEGMENTUV)
    int curSegmentionU = int(cSegmentUVNum.x * goalUoffsetNormal);
    int curSegmentionV = int(cSegmentUVNum.y * goalVoffsetNormal);
    vGoalUVoffset = vec2(float(curSegmentionU)/cSegmentUVNum.x*uvSize.x,float(curSegmentionV)/cSegmentUVNum.y*uvSize.y);
#else
    vGoalUVoffset = vec2( goalUoffsetNormal * uvSize.x, goalVoffsetNormal * uvSize.y);
#endif
#endif
    

}

void PS()
{
#if defined(FREQUENCYUVOFFSET)
    vec2 finalTexC = vec2(vTexCoord.xy+vGoalUVoffset.xy);
#elif defined(CLOUD)
    
#else
    vec2 finalTexC = vTexCoord.xy;
#endif
    
#if defined(CLOUD)
    vec4 diffInput = vec4(1.0,1.0,1.0,1.0);
#elif defined(STATIONARY_ICON)
    vec4 diffInput = vec4(1.0,1.0,1.0,1.0);
#elif defined(MCDEBUG)
    vec4 diffInput = vec4(1.0,1.0,1.0,1.0);
#else
    vec4 diffInput = texture2D(sDiffMap, finalTexC.xy);
#endif
    
#ifdef ALPHAMASK
    if (diffInput.a < 0.01)
        discard;
#endif

    // Urho3D Fog Start
    #ifdef HEIGHTFOG
        float fogFactor = GetHeightFogFactor(vWorldPos.w, vWorldPos.y);
    #else
        float fogFactor = GetFogFactor(vWorldPos.w);
    #endif
    // Urho3D Fog End

#if defined(CLOUD)

#elif defined(STATIONARY_RAIN)
//    diffInput.rgb=pow(diffInput.rgb, GammaV3);
#else
//    diffInput.rgb=pow(diffInput.rgb, GammaV3);
#endif
    
#if defined(CLOUD)

#elif defined(STATIONARY_RAIN)
    
#else
    //SoSo by yuxiaolong
    vec4 MCSpecInput = vec4(1.0,1.0,1.0,1.0);
    if (vMCSpecFactor != vec3(0,0,0)) 
         MCSpecInput = texture2D(sSpecMap, vec2(vMCSpecFactor.x*finalTexC.x+vOffsetSpec,vMCSpecFactor.y*finalTexC.y+vOffsetSpec));
    vec4 MCSpecColor =  vMCSpecColor*MCSpecInput;
    //SoSo by yuxiaolong
    //sSpecMap
#endif
    vec4 ambientV4=vec4(1,1,1,1);
//    vec4 ambientV4=vec4(cAmbientColor,1);
#if defined(FIRE)
    vec4 diffColor = ambientV4 * diffInput;
#elif defined(CREAK)
    vec4 diffColor = ambientV4 * diffInput;
    diffColor.rgb= diffColor.rgb + vec3(0.4);
#elif defined(LAVA)
    vec4 diffColor = vColor * diffInput;
#elif defined(STATIONARY_BLOCK)
    vec4 diffColor = ambientV4 * diffInput * MCSpecInput * vColor;
#elif defined(STATIONARY_GRASS)
    vec4 diffColor = ambientV4 * diffInput;
#elif defined(STATIONARY_LEAVES)
    vec4 diffColor = ambientV4 * diffInput;
#elif defined(STATIONARY_BOXEFF)
    vec4 diffColor = ambientV4 * cMatDiffColor * diffInput;
#elif defined(STATIONARY_EXP)
    vec4 diffColor = ambientV4 * diffInput;
#elif defined(CLOUD)
    vec4 diffColor = ambientV4 * vColor;
#elif defined(STATIONARY_ICON)
    vec4 diffColor = ambientV4 * vColor * MCSpecInput;
#elif defined(STATIONARY_RAIN)
    vec4 diffColor = ambientV4 * diffInput;
#elif defined(STATIONARY_BEACON)
    vec4 diffColor = ambientV4 * vColor * diffInput;
#elif defined(STATIONARY_SHADOW)
    vec4 diffColor = ambientV4 * diffInput;
#elif defined(STATIONARY_GLASS)
    vec4 diffColor = ambientV4 * diffInput * MCSpecInput;
#elif defined(MCDEBUG)
    vec4 diffColor = ambientV4 * vColor;
#else
    vec4 diffColor = ambientV4 * vColor * diffInput;
#endif

    vec3 finalColor=diffColor.rgb;
    #ifdef AMBIENT
        finalColor += cAmbientColor * diffColor.rgb * 0.1;
  //      finalColor += cMatEmissiveColor;
        gl_FragColor = vec4(GetFog(finalColor, fogFactor), diffColor.a);
    #else
        gl_FragColor = vec4(GetLitFog(finalColor, fogFactor), diffColor.a);
    #endif
}
