
#include "Uniforms.glsl"
#include "Samplers.glsl"
#include "Transform.glsl"
#include "MCHeader.glsl"

#ifdef DIFFCUBEMAP
varying vec3 vTexCoord;
#else
varying vec2 vTexCoord;
#endif

#ifdef UVOFFSET
uniform vec2 cUVOffset;
#endif



void VS()
{
    mat4 modelMatrix = iModelMatrix;
    vec3 worldPos = GetWorldPos(modelMatrix);
    gl_Position = GetClipPos(worldPos);
#ifndef GL_ES
    gl_Position.z = gl_Position.w;
#else
    // On OpenGL ES force Z slightly in front of far plane to avoid clipping artifacts due to inaccuracy
    gl_Position.z = 0.99999 * gl_Position.w;
#endif
    
    vTexCoord = iTexCoord;
    
#ifdef UVOFFSET
    vTexCoord.x  = cUVOffset.x + vTexCoord.x;
    vTexCoord.y  = cUVOffset.y + vTexCoord.y;
#endif
    
}

void PS()
{
    
#ifdef DIFFMAP
    vec4 diffInput = texture2D(sDiffMap, vTexCoord);
#elif DIFFCUBEMAP
    vec4 diffInput = textureCube(sDiffCubeMap, vTexCoord);
#else
    vec4 diffInput = vec4(1.0,1.0,1.0,1.0);
#endif
    
    gl_FragColor = diffInput;
}


