#include "Uniforms.glsl"
#include "Samplers.glsl"
#include "Transform.glsl"
#include "ScreenPos.glsl"
#include "Fog.glsl"
#include "MCHeader.glsl"
varying vec4 vScreenPos;
varying vec2 vReflectUV;
varying vec2 vWaterUV;
varying vec3 vNormal;
varying vec4 vEyeVec;

#ifdef DEVELOPING
varying vec4 vColor;
#endif
#ifdef COMPILEVS
uniform vec2 cNoiseSpeed;
uniform float cNoiseTiling;
#endif
#ifdef COMPILEPS
uniform float cNoiseStrength;
uniform float cFresnelPower;
uniform vec3 cWaterTint;
#endif


//SoSo by yuxiaolong define a fog color
varying vec4 fogColor;
varying  float vOffsetSpec;
varying vec4 vMCSpecColor;
varying vec3 vMCSpecFactor;
//soso by yuxiaolong

const vec3 UNIT_Y = vec3(0,1,0);
const float DIST_DESATURATION = 56.0 / 255.0; //WARNING this value is also hardcoded in the water color, don'tchange
const float FAR_CHUNKS_DISTANCE=96.0;
void VS()
{
    mat4 modelMatrix = iModelMatrix;
    vec3 worldPos = GetWorldPos(modelMatrix);
    gl_Position = GetClipPos(worldPos);
    vScreenPos = GetScreenPos(gl_Position);
    // GetQuadTexCoord() returns a vec2 that is OK for quad rendering; multiply it with output W
    // coordinate to make it work with arbitrary meshes such as the water plane (perform divide in pixel shader)
    // Also because the quadTexCoord is based on the clip position, and Y is flipped when rendering to a texture
    // on OpenGL, must flip again to cancel it out
    vReflectUV = GetQuadTexCoord(gl_Position);
    vReflectUV.y = 1.0 - vReflectUV.y;
    vReflectUV *= gl_Position.w;
    vWaterUV = iTexCoord * cNoiseTiling + cElapsedTime * cNoiseSpeed;
    //hopjoy by yuxiaolong
    fogColor = MCGetFogColor(worldPos,cCameraPos);
    
    //input MCSpecColor
    vMCSpecColor = MCGetSpecColor();
    //influence (x y )   ->  coordinates scale z offset circle
    vMCSpecFactor = MCGetSpecFactor();
    vOffsetSpec = fract(cElapsedTime/vMCSpecFactor.z);
    //hopjoy by yuxiaolong
    //vNormal = GetWorldNormal(modelMatrix); HOPJOY
    vNormal = normalize(cCameraPos-worldPos); //HOPJOY
    
    vEyeVec = vec4(cCameraPos - worldPos, GetDepth(gl_Position));

#ifdef DEVELOPING
    vColor=vec4(0.5);
    float cameraDepth = worldPos.z;

    float F = dot(normalize(vNormal), UNIT_Y);                                          
    F = 1.0 - max(F, 0.1);
    F = 1.0 - mix(F*F*F*F, 1.0, min(1.0, cameraDepth / FAR_CHUNKS_DISTANCE));

    vColor.rg -= vec2(F * DIST_DESATURATION);

    vec4 depthColor = vec4(vColor.rgb * 0.5, 1.0);
    vec4 traspColor = vec4(vColor.rgb * 0.45, 0.8);
    vec4 surfColor = vec4(vColor.rgb, 1.0);

    vec4 nearColor = mix(traspColor, depthColor, vColor.a);
    vColor = mix(surfColor, nearColor, F);
#endif

}

void PS()
{
    vec2 refractUV = vScreenPos.xy / vScreenPos.w;
    vec2 reflectUV = vReflectUV.xy / vScreenPos.w;

    vec2 noise = (texture2D(sNormalMap, vWaterUV).rg - 0.5) * cNoiseStrength;
    refractUV += noise;
    // Do not shift reflect UV coordinate upward, because it will reveal the clipping of geometry below water
    if (noise.y < 0.0)
        noise.y = 0.0;
    reflectUV += noise;

    float fresnel = pow(1.0 - clamp(dot(normalize(vEyeVec.xyz), vNormal), 0.0, 1.0), cFresnelPower);
    vec3 refractColor = texture2D(sEnvMap, refractUV).rgb * cWaterTint;
    vec3 reflectColor = texture2D(sDiffMap, reflectUV).rgb;
    vec3 finalColor = mix(refractColor, reflectColor, fresnel);
    //SoSo by yuxiaolong
    if (fogColor.a>0.975)
        discard;
    
    finalColor = mix(refractColor, reflectColor, fresnel)*cMatDiffColor.rgb;
    
    vec4 finalC = mix(vec4(GetFog(finalColor, GetFogFactor(vEyeVec.w)), 1.0), fogColor, fogColor.a);
    vec4 MCSpecInput = texture2D(sSpecMap, vec2(vMCSpecFactor.x*reflectUV.x+vOffsetSpec,vMCSpecFactor.y*reflectUV.y+vOffsetSpec));
    vec4 MCSpecColor =  vMCSpecColor*MCSpecInput;
    finalC  =  mix(finalC, MCSpecColor, MCSpecColor.a);
    //SoSo by yuxiaolong
    gl_FragColor =finalC ;
}
