#include "Uniforms.glsl"
#include "Samplers.glsl"
#include "Transform.glsl"
#include "ScreenPos.glsl"
#include "Lighting.glsl"
#include "Fog.glsl"

#ifdef NORMALMAP
    varying vec4 vTexCoord;
    varying vec4 vTangent;
#else
    varying vec2 vTexCoord;
#endif
varying vec3 vNormal;
varying vec4 vWorldPos;
#ifdef PERPIXEL
    #ifdef SHADOW
        varying vec4 vShadowPos[NUMCASCADES];
    #endif
    #ifdef SPOTLIGHT
        varying vec4 vSpotPos;
    #endif
    #ifdef POINTLIGHT
        varying vec3 vCubeMaskVec;
    #endif
#else
    varying vec3 vVertexLight;
    varying vec4 vScreenPos;
    #ifdef ENVCUBEMAP
        varying vec3 vReflectionVec;
    #endif
    #if defined(LIGHTMAP) || defined(AO)
        varying vec2 vTexCoord2;
    #endif
#endif



const float pi = 3.14159265;
// 这东西似乎需要改改，灯光方向可能是传入的？
const vec3 light_direction = normalize(vec3(-1.0, 1.0, -1.0));

//varying vec2 vTexCoord;
varying float vFragment_ao;
varying float vFragment_light;
varying float vFog_factor;
varying float vFog_height;
varying float vDiffuse;

void VS()
{
    mat4 modelMatrix = iModelMatrix;
    vec3 worldPos = GetWorldPos(modelMatrix);
    gl_Position = GetClipPos(worldPos);
    vNormal = GetWorldNormal(modelMatrix);
    vWorldPos = vec4(worldPos, GetDepth(gl_Position));


    vTexCoord = GetTexCoord(iTexCoord);
    vFragment_ao = 0.3 + (1.0 - iTexCoord2.x) * 0.7;
    vFragment_light = iTexCoord2.y;
    vDiffuse = max(0.0, dot(iNormal, light_direction));

    vFog_factor = 0.0;
    vFog_height = 0.0;
}

void PS()
{
    // 目前生成的UV贴图，Y坐标是反的...
    vec2 uv = vec2(vTexCoord.x,1.0-vTexCoord.y);
    //vec2 uv = vTexCoord;
    vec3 diffInput = vec3(texture2D(sDiffMap, uv));
    //vec3 diffColor = cMatDiffColor * diffInput;
    // 处理透明色
    if (diffInput == vec3(1.0, 0.0, 1.0)) {
       discard;
    }
    bool cloud = diffInput == vec3(1.0, 1.0, 1.0);

    float df = cloud ? 1.0 - vDiffuse * 0.2 : vDiffuse;
    float ao = cloud ? 1.0 - (1.0 - vFragment_ao) * 0.2 : vFragment_ao;
    ao = min(1.0, ao + vFragment_light);
    df = min(1.0, df + vFragment_light);
    //TODO: 传入uniform: daylight,或者计算出daylight
    float daylight = 0.5;
    float value = min(1.0, daylight + vFragment_light);
    vec3 light_color = vec3(value * 0.3 + 0.2);
    vec3 ambient = vec3(value * 0.3 + 0.2);
    vec3 light = ambient + light_color * df;
    diffInput = clamp(diffInput * light * ao, vec3(0.0), vec3(1.0));
    //TODO: 混合天空的颜色
    //vec3 sky_color = vec3(texture2D(sky_sampler, vec2(timer, vFog_height)));
    //diffInput = mix(diffInput, sky_color, vFog_factor);
    //
    gl_FragColor = vec4(diffInput, 1.0);

}
